@@ -24,9 +24,6 @@
 #endif /* HAVE_CONFIG_H */
 #include <stdio.h>
 #include <stdlib.h>
-#ifdef HAVE_GETOPT_H
-#include <getopt.h>
-#endif /* HAVE_GETOPT_H */
 #include <locale.h>
 #include <string.h>
 #include <sys/types.h>
@@ -34,8 +31,9 @@
 #include <fcntl.h>
 #include <errno.h>
 #include <assert.h>
-#include <unistd.h>
 #include <limits.h>
+#include <unistd.h>
+#include "getopt.h"
 #include "regex.h"
 #include "gettext.h"
 #define _(String) gettext(String)
@@ -47,12 +45,11 @@
 
 /* Short options. */
 static char const short_options[] =
-"cd:Me:fhHiklnsvwBD:E:I:S:yV0123456789";
+"cd:e:hiklnqsvwyBD:E:HI:MS:V0123456789";
 
 static int show_help;
 char *program_name;
 
-#ifdef HAVE_GETOPT_LONG
 /* Long options that have no corresponding short equivalents. */
 enum {
   COLOR_OPTION = CHAR_MAX + 1,
@@ -62,34 +59,35 @@ enum {
 /* Long option equivalences. */
 static struct option const long_options[] =
 {
+  {"best-match", no_argument, NULL, 'B'},
+  {"color", no_argument, NULL, COLOR_OPTION},
+  {"colour", no_argument, NULL, COLOR_OPTION},
   {"count", no_argument, NULL, 'c'},
+  {"delete-cost", required_argument, NULL, 'D'},
   {"delimiter", no_argument, NULL, 'd'},
   {"delimiter-after", no_argument, NULL, 'M'},
-  {"regexp", required_argument, NULL, 'e'},
-  {"no-filename", no_argument, NULL, 'h'},
-  {"with-filename", no_argument, NULL, 'H'},
-  {"ignore-case", no_argument, NULL, 'i'},
-  {"literal", no_argument, NULL, 'k'},
   {"files-with-matches", no_argument, NULL, 'l'},
-  {"record-number", no_argument, NULL, 'n'},
-  {"line-number", no_argument, NULL, 'n'},
-  {"show-cost", no_argument, NULL, 's'},
-  {"invert-match", no_argument, NULL, 'v'},
-  {"word-regexp", no_argument, NULL, 'w'},
-  {"best-match", no_argument, NULL, 'B'},
-  {"delete-cost", required_argument, NULL, 'D'},
-  {"max-errors", required_argument, NULL, 'E'},
-  {"insert-cost", required_argument, NULL, 'I'},
-  {"substitute-cost", required_argument, NULL, 'S'},
-  {"nothing", no_argument, NULL, 'y'},
-  {"version", no_argument, NULL, 'V'},
   {"help", no_argument, &show_help, 1},
-  {"colour", no_argument, NULL, COLOR_OPTION},
-  {"color", no_argument, NULL, COLOR_OPTION},
+  {"ignore-case", no_argument, NULL, 'i'},
+  {"insert-cost", required_argument, NULL, 'I'},
+  {"invert-match", no_argument, NULL, 'v'},
+  {"line-number", no_argument, NULL, 'n'},
+  {"literal", no_argument, NULL, 'k'},
+  {"max-errors", required_argument, NULL, 'E'},
+  {"no-filename", no_argument, NULL, 'h'},
+  {"nothing", no_argument, NULL, 'y'},
+  {"quiet", no_argument, NULL, 'q'},
+  {"record-number", no_argument, NULL, 'n'},
+  {"regexp", required_argument, NULL, 'e'},
+  {"show-cost", no_argument, NULL, 's'},
   {"show-position", no_argument, NULL, SHOW_POSITION_OPTION},
+  {"silent", no_argument, NULL, 'q'},
+  {"substitute-cost", required_argument, NULL, 'S'},
+  {"version", no_argument, NULL, 'V'},
+  {"with-filename", no_argument, NULL, 'H'},
+  {"word-regexp", no_argument, NULL, 'w'},
   {0, 0, 0, 0}
 };
-#endif /* HAVE_GETOPT_LONG */
 
 static void
 tre_agrep_usage(int status)
@@ -129,7 +127,7 @@ Miscellaneous:\n\
   -d, --delimiter=PATTERN   set the record delimiter regular expression\n\
   -v, --invert-match	    select non-matching records\n\
   -V, --version		    print version information and exit\n\
-  -y			    does nothing (for compatibility with the non-free\n\
+  -y, --nothing		    does nothing (for compatibility with the non-free\n\
 			    agrep program)\n\
       --help		    display this help and exit\n\
 \n\
@@ -139,14 +137,19 @@ Output control:\n\
   -h, --no-filename	    suppress the prefixing filename on output\n\
   -H, --with-filename	    print the filename for each match\n\
   -l, --files-with-matches  only print FILE names containing matches\n\
+  -M, --delimiter-after     print record delimiter after record if -d is used\n\
   -n, --record-number	    print record number with output\n\
+      --line-number         same as -n\n\
+  -q, --quiet, --silent	    suppress all normal output\n\
   -s, --show-cost	    print match cost with output\n\
       --colour, --color     use markers to distinguish the matching \
-strings\n"));
+strings\n\
+      --show-position       prefix each output record with start and end\n\
+                            position of the first match within the record\n"));
       printf("\n");
       printf(_("\
-With no FILE, or when FILE is -, reads standard input.	If less than two\n\
-FILEs are given, -h is assumed.	 Exit status is 0 if a match is found, 1 for\n\
+With no FILE, or when FILE is -, reads standard input.  If less than two\n\
+FILEs are given, -h is assumed.  Exit status is 0 if a match is found, 1 for\n\
 no match, and 2 if there were errors.  If -E or -# is not specified, only\n\
 exact matches are selected.\n"));
       printf("\n");
@@ -154,7 +157,8 @@ exact matches are selected.\n"));
 PATTERN is a POSIX extended regular expression (ERE) with the TRE extensions.\n\
 See tre(7) for a complete description.\n"));
       printf("\n");
-      printf(_("Report bugs to Ville Laurikari <vl@iki.fi>.\n"));
+      printf(_("Report bugs to: "));
+      printf("%s.\n", PACKAGE_BUGREPORT);
     }
   exit(status);
 }
@@ -168,10 +172,12 @@ static int buf_size;	   /* Current size of the buffer. */
 static int data_len;	   /* Amount of data in the buffer. */
 static char *record;	   /* Start of current record. */
 static char *next_record;  /* Start of next record. */
-static char *next_search;
 static int record_len;	   /* Length of current record. */
-static int delim_after = 1;
+static int delim_len;      /* Length of delimiter before record. */
+static int next_delim_len; /* Length of delimiter after record. */
+static int delim_after = 1;/* If true, print the delimiter after the record. */
 static int at_eof;
+static int have_matches;   /* If true, matches have been found. */
 
 static int invert_match;   /* Show only non-matching records. */
 static int print_filename; /* Output filename. */
@@ -184,6 +190,7 @@ static int print_position;  /* Show start and end offsets for matches. */
 
 static int best_match;	     /* Output only best matches. */
 static int best_cost;	     /* Best match cost found so far. */
+static int be_silent;	     /* Never output anything */
 
 static regaparams_t match_params;
 
@@ -195,22 +202,50 @@ static char *highlight = "01;31";
    to the length of the record.	 Returns 1 when there are no more records,
    0 otherwise. */
 static inline int
-tre_agrep_get_next_record(int fd)
+tre_agrep_get_next_record(int fd, const char *filename)
 {
-  int errcode;
-  regmatch_t pmatch[1];
-
   if (at_eof)
     return 1;
 
   while (1)
     {
-      if (next_search == NULL)
+      int errcode;
+      regmatch_t pmatch[1];
+
+      if (next_record == NULL)
 	{
 	  int r;
-	  /* Fill the buffer with data from the file. */
-	  r = read(fd, buf + data_len, buf_size - data_len);
-	  /* XXX - check for failure or interruption. */
+	  int read_size = buf_size - data_len;
+
+	  if (read_size <= 0)
+	    {
+	      /* The buffer is full and no record delimiter found yet,
+		 we need to grow the buffer.  We double the size to
+		 avoid rescanning the data too many times when the
+		 records are very large. */
+	      buf_size *= 2;
+	      buf = realloc(buf, buf_size);
+	      if (buf == NULL)
+		{
+		  fprintf(stderr, "%s: %s\n", program_name, _("Out of memory"));
+		  exit(2);
+		}
+	      read_size = buf_size - data_len;
+	    }
+
+	  r = read(fd, buf + data_len, read_size);
+	  if (r < 0)
+	    {
+	      /* Read error. */
+	      char *err;
+	      if (errno == EINTR)
+		continue;
+	      err = strerror(errno);
+	      fprintf(stderr, "%s: ", program_name);
+	      fprintf(stderr, _("Error reading from %s: %s\n"), filename, err);
+	      return 1;
+	    }
+
 	  if (r == 0)
 	    {
 	      /* End of file.  Return the last record. */
@@ -221,60 +256,55 @@ tre_agrep_get_next_record(int fd)
 		 to be a record. */
 	      if (record_len == 0)
 		return 1;
-	      else
-		return 0;
+	      return 0;
 	    }
 	  data_len += r;
-	  next_search = next_record = buf;
+	  next_record = buf;
 	}
 
       /* Find the next record delimiter. */
-      errcode = regnexec(&delim, next_search, data_len - (next_search - buf),
+      errcode = regnexec(&delim, next_record, data_len - (next_record - buf),
 			 1, pmatch, 0);
-      if (errcode == REG_ESPACE)
-	{
-	  fprintf(stderr, "%s: %s\n", program_name, _("Out of memory"));
-	  exit(2);
-	}
-      else if (errcode == REG_NOMATCH)
-	{
-	  /* No record delimiter found. */
-	  if (next_search == buf)
-	    {
-	      /* The buffer is full but we don't yet have a full record.
-		 Grow the buffer. */
-	      buf = realloc(buf, buf_size * 2);
-	      if (!buf)
-		{
-		  fprintf(stderr, "%s: %s\n", program_name,
-			  _("Out of memory"));
-		  exit(2);
-		}
-	      buf_size *= 2;
-	      next_search = NULL;
-	    }
-	  else
-	    {
-	      /* Move the data to start of the buffer and read more data. */
-	      memmove(buf, next_search, buf + data_len - next_search);
-	      data_len = buf + data_len - next_search;
-	      next_search = NULL;
-	    }
-	}
-      else if (errcode == REG_OK)
+
+
+      switch (errcode)
 	{
+	case REG_OK:
 	  /* Record delimiter found, now we know how long the current
 	     record is. */
 	  record = next_record;
-	  if (delim_after)
-	    next_record = next_search + pmatch[0].rm_eo;
-	  else
-	    next_record = next_search + pmatch[0].rm_so;
-	  record_len = next_record - record;
-	  next_search = next_search + pmatch[0].rm_eo;
+	  record_len = pmatch[0].rm_so;
+	  delim_len = next_delim_len;
+
+	  next_delim_len = pmatch[0].rm_eo - pmatch[0].rm_so;
+	  next_record = next_record + pmatch[0].rm_eo;
 	  return 0;
+	  break;
+
+	case REG_NOMATCH:
+	  if (next_record == buf)
+	    {
+	      next_record = NULL;
+	      continue;
+	    }
+
+	  /* Move the data to start of the buffer and read more
+	     data. */
+	  memmove(buf, next_record, buf + data_len - next_record);
+	  data_len = buf + data_len - next_record;
+	  next_record = NULL;
+	  continue;
+	  break;
+
+	case REG_ESPACE:
+	  fprintf(stderr, "%s: %s\n", program_name, _("Out of memory"));
+	  exit(2);
+	  break;
+
+	default:
+	  assert(0);
+	  break;
 	}
-      else assert(0);
     }
 }
 
@@ -299,7 +329,7 @@ tre_agrep_handle_file(const char *filename)
     }
 
   /* Reset read buffer state. */
-  next_search = NULL;
+  next_record = NULL;
   data_len = 0;
 
   if (!filename || strcmp(filename, "-") == 0)
@@ -328,7 +358,7 @@ tre_agrep_handle_file(const char *filename)
   /* Go through all records and output the matching ones, or the non-matching
      ones if `invert_match' is true. */
   at_eof = 0;
-  while (!tre_agrep_get_next_record(fd))
+  while (!tre_agrep_get_next_record(fd, filename))
     {
       int errcode;
       regamatch_t match;
@@ -352,7 +382,11 @@ tre_agrep_handle_file(const char *filename)
       if ((!invert_match && errcode == REG_OK)
 	  || (invert_match && errcode == REG_NOMATCH))
 	{
+	  if (be_silent)
+	    exit(0);
+
 	  count++;
+	  have_matches = 1;
 	  if (best_match)
 	    {
 	      if (best_match == 1)
@@ -381,16 +415,32 @@ tre_agrep_handle_file(const char *filename)
 	      if (print_cost)
 		printf("%d:", match.cost);
 	      if (print_position)
-		printf("%d-%d:", pmatch[0].rm_so, pmatch[0].rm_eo);
+		printf("%d-%d:",
+		       invert_match ? 0 : (int)pmatch[0].rm_so,
+		       invert_match ? record_len : (int)pmatch[0].rm_eo);
 
-	      if (color_option)
+	      /* Adjust record boundaries so we print the delimiter
+		 before or after the record. */
+	      if (delim_after)
 		{
-		  printf("%.*s", pmatch[0].rm_so, record);
+		  record_len += next_delim_len;
+		}
+	      else
+		{
+		  record -= delim_len;
+		  record_len += delim_len;
+		  pmatch[0].rm_so += delim_len;
+		  pmatch[0].rm_eo += delim_len;
+		}
+
+	      if (color_option && !invert_match)
+		{
+		  printf("%.*s", (int)pmatch[0].rm_so, record);
 		  printf("\33[%sm", highlight);
-		  printf("%.*s", pmatch[0].rm_eo - pmatch[0].rm_so,
+		  printf("%.*s", (int)(pmatch[0].rm_eo - pmatch[0].rm_so),
 			 record + pmatch[0].rm_so);
 		  fputs("\33[00m", stdout);
-		  printf("%.*s", record_len - pmatch[0].rm_eo,
+		  printf("%.*s", (int)(record_len - pmatch[0].rm_eo),
 			 record + pmatch[0].rm_eo);
 		}
 	      else
@@ -401,7 +451,7 @@ tre_agrep_handle_file(const char *filename)
 	}
     }
 
-  if (count_matches && !best_match)
+  if (count_matches && !best_match && !be_silent)
     {
       if (print_filename)
 	printf("%s:", filename);
@@ -444,22 +494,14 @@ main(int argc, char **argv)
   /* Defaults. */
   print_filename = -1;
   print_cost = 0;
+  be_silent = 0;
   regaparams_default(&match_params);
   match_params.max_cost = 0;
 
-#ifndef HAVE_GETOPT_LONG
-  if (argc == 2 && strcmp(argv[1], "--help") == 0)
-    tre_agrep_usage(0);
-#endif /* !HAVE_GETOPT_LONG */
-
   /* Parse command line options. */
   while (1)
     {
-#ifdef HAVE_GETOPT_LONG
       c = getopt_long(argc, argv, short_options, long_options, NULL);
-#else /* !HAVE_GETOPT_LONG */
-      c = getopt(argc, argv, short_options);
-#endif /* !HAVE_GETOPT_LONG */
       if (c == -1)
 	break;
 
@@ -475,30 +517,22 @@ main(int argc, char **argv)
 	  if (delim_after == 1)
 	    delim_after = 0;
 	  break;
-	case 'M':
-	  /* Print delimiters after matches instead of before. */
-	  delim_after = 2;
-	  break;
 	case 'e':
 	  /* Regexp to use. */
 	  regexp = optarg;
 	  break;
-	case 'k':
-	  /* The pattern is a literal string. */
-	  literal_string = 1;
-	  break;
 	case 'h':
 	  /* Don't prefix filename on output if there are multiple files. */
 	  print_filename = 0;
 	  break;
-	case 'H':
-	  /* Always print filename prefix on output. */
-	  print_filename = 1;
-	  break;
 	case 'i':
 	  /* Ignore case. */
 	  comp_flags |= REG_ICASE;
 	  break;
+	case 'k':
+	  /* The pattern is a literal string. */
+	  literal_string = 1;
+	  break;
 	case 'l':
 	  /* Only print files that contain matches. */
 	  list_files = 1;
@@ -507,6 +541,9 @@ main(int argc, char **argv)
 	  /* Print record number of matching record. */
 	  print_recnum = 1;
 	  break;
+	case 'q':
+	  be_silent = 1;
+	  break;
 	case 's':
 	  /* Print match cost of matching record. */
 	  print_cost = 1;
@@ -535,10 +572,18 @@ main(int argc, char **argv)
 	  match_params.max_cost = atoi(optarg);
 	  max_cost_set = 1;
 	  break;
+	case 'H':
+	  /* Always print filename prefix on output. */
+	  print_filename = 1;
+	  break;
 	case 'I':
 	  /* Set the cost of an insertion. */
 	  match_params.cost_ins = atoi(optarg);
 	  break;
+	case 'M':
+	  /* Print delimiters after matches instead of before. */
+	  delim_after = 2;
+	  break;
 	case 'S':
 	  /* Set the cost of a substitution. */
 	  match_params.cost_subst = atoi(optarg);
@@ -561,14 +606,12 @@ PARTICULAR PURPOSE.\n"));
 	case '?':
 	  /* Ambiguous match or extraneous parameter. */
 	  break;
-#ifdef HAVE_GETOPT_LONG
 	case COLOR_OPTION:
 	  color_option = 1;
 	  break;
 	case SHOW_POSITION_OPTION:
 	  print_position = 1;
 	  break;
-#endif /* HAVE_GETOPT_LONG */
 	case 0:
 	  /* Long options without corresponding short options. */
 	  break;
@@ -755,5 +798,5 @@ PARTICULAR PURPOSE.\n"));
 	tre_agrep_handle_file(argv[optind++]);
     }
 
-  return 0;
+  return have_matches == 0;
 }
