@@ -72,7 +72,7 @@ static int exec_errors = 0;
 
 #define END -2
 
-void
+static void
 test_status(char c)
 {
   static int k = 0;
@@ -201,8 +201,8 @@ check(va_list ap, int ret, char *str, size_t pmatch_len, regmatch_t *pmatch,
 	      printf("	group %d: expected (%d, %d) \"%.*s\", "
 		     "got (%d, %d) \"%.*s\"\n",
 		     i, rm_so, rm_eo, rm_eo - rm_so, str + rm_so,
-		     pmatch[i].rm_so, pmatch[i].rm_eo,
-		     pmatch[i].rm_eo - pmatch[i].rm_so,
+		     (int)pmatch[i].rm_so, (int)pmatch[i].rm_eo,
+		     (int)(pmatch[i].rm_eo - pmatch[i].rm_so),
 		     str + pmatch[i].rm_so);
 	      fail = 1;
 	    }
@@ -224,7 +224,7 @@ check(va_list ap, int ret, char *str, size_t pmatch_len, regmatch_t *pmatch,
 	      printf("Exec error, regex: \"%s\", string: \"%s\"\n",
 		     regex_pattern, str);
 	    printf("  group %d: expected (-1, -1), got (%d, %d)\n",
-		   i, pmatch[i].rm_so, pmatch[i].rm_eo);
+		   i, (int)pmatch[i].rm_so, (int)pmatch[i].rm_eo);
 	    fail = 1;
 	  }
     }
@@ -335,7 +335,7 @@ test_exec(char *str, int eflags, ...)
 }
 
 
-void
+static void
 test_comp(char *re, int flags, int ret)
 {
   int errcode = 0;
@@ -727,6 +727,14 @@ main(int argc, char **argv)
   test_comp("(Ab|cD)*", REG_EXTENDED | REG_ICASE, 0);
   test_exec("aBcD", 0, REG_OK, 0, 4, 2, 4, END);
 
+  test_comp("a**", REG_EXTENDED, REG_BADRPT);
+  test_comp("a*+", REG_EXTENDED, REG_BADRPT);
+  test_comp("a+*", REG_EXTENDED, REG_BADRPT);
+  test_comp("a++", REG_EXTENDED, REG_BADRPT);
+  test_comp("a?+", REG_EXTENDED, REG_BADRPT);
+  test_comp("a?*", REG_EXTENDED, REG_BADRPT);
+  test_comp("a{1,2}*", REG_EXTENDED, REG_BADRPT);
+  test_comp("a{1,2}+", REG_EXTENDED, REG_BADRPT);
 
   /*
    * Many of the following tests were mostly inspired by (or copied from) the
@@ -769,6 +777,11 @@ main(int argc, char **argv)
   test_exec("a", 0, REG_OK, 0, 1, END);
   test_exec("\n", 0, REG_OK, 0, 1, END);
 
+  test_comp("(+|?)", 0, 0);
+  test_exec("(+|?)", 0, REG_OK, 0, 5, END);
+  test_exec("+|?", 0, REG_NOMATCH);
+  test_exec("(+)", 0, REG_NOMATCH);
+  test_exec("+", 0, REG_NOMATCH);
 
 
   /*
@@ -1112,7 +1125,10 @@ main(int argc, char **argv)
   /* Shorthands for character classes. */
   test_comp("\\w+", REG_EXTENDED, 0);
   test_exec(",.(a23_Nt-o)", 0, REG_OK, 3, 9, END);
-  test_comp("\\D", REG_EXTENDED, 0);
+  test_comp("\\d+", REG_EXTENDED, 0);
+  test_exec("uR120_4=v4", 0, REG_OK, 2, 5, END);
+  test_comp("\\D+", REG_EXTENDED, 0);
+  test_exec("120d_=vA4s", 0, REG_OK, 3, 8, END);
 
   /* Quoted special characters. */
   test_comp("\\t", REG_EXTENDED, 0);
@@ -1410,7 +1426,7 @@ main(int argc, char **argv)
   test_exec("abcd", 0, REG_OK, 0, 4, END);
 
   avoid_eflags = 0;
-  
+
 
   /*
    * Error reporting tests.
@@ -1452,7 +1468,7 @@ main(int argc, char **argv)
   test_comp("+", REG_EXTENDED, 0);
   test_exec("ξޤϡ", 0, REG_OK, 10, 13, END);
 
-#ifndef WIN32
+#if !defined(WIN32) && !defined(__OpenBSD__)
   if (setlocale(LC_CTYPE, "fi_FI.ISO-8859-1") != NULL)
     {
       printf("\nTesting LC_CTYPE fi_FI.ISO-8859-1\n");
@@ -1476,7 +1492,7 @@ main(int argc, char **argv)
       test_exec("abc", 0, REG_OK, 2, 3, END);
     }
 #endif /* TRE_MULTIBYTE */
-#endif /* !WIN32 */
+#endif
 
   regfree(&reobj);
 
